/*
 * FILE:
 *   dx11_gfx_context.hpp
 *
 * PURPOSE:
 *   DirectX 11 implementation of GfxContext
 *
 * AUTHOR:
 *   Stephen Thompson <stephen@solarflare.org.uk>
 *
 * CREATED:
 *   20-Oct-2011
 *   
 * COPYRIGHT:
 *   Copyright (C) Stephen Thompson, 2008 - 2011.
 *
 *   This file is part of the "Coercri" software library. Usage of "Coercri"
 *   is permitted under the terms of the Boost Software License, Version 1.0, 
 *   the text of which is displayed below.
 *
 *   Boost Software License - Version 1.0 - August 17th, 2003
 *
 *   Permission is hereby granted, free of charge, to any person or organization
 *   obtaining a copy of the software and accompanying documentation covered by
 *   this license (the "Software") to use, reproduce, display, distribute,
 *   execute, and transmit the Software, and to prepare derivative works of the
 *   Software, and to permit third-parties to whom the Software is furnished to
 *   do so, all subject to the following:
 *
 *   The copyright notices in the Software and this entire statement, including
 *   the above license grant, this restriction and the following disclaimer,
 *   must be included in all copies of the Software, in whole or in part, and
 *   all derivative works of the Software, unless such copies or derivative
 *   works are solely in the form of machine-executable object code generated by
 *   a source language processor.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 *   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 */

#ifndef DX11_GFX_CONTEXT_HPP
#define DX11_GFX_CONTEXT_HPP

#include "../../gfx/gfx_context.hpp"

#include <d3d11.h>
#ifdef min
#undef min
#endif
#ifdef max
#undef max
#endif

namespace Coercri {

    class PrimitiveBatch;
    
    class DX11GfxContext : public GfxContext {
    public:
        // ctor not intended to be called directly. Use Window::createGfxContext.
        DX11GfxContext(ID3D11Device *pDevice,
                       ID3D11DeviceContext *pDeviceContext,
                       ID3D11RenderTargetView *pRenderTargetView,
                       IDXGISwapChain *pSwapChain,
                       PrimitiveBatch &prim_batch);
        virtual ~DX11GfxContext();

        // overridden from GfxContext
        void setClipRectangle(const Rectangle &rect);
        void clearClipRectangle();
        Rectangle getClipRectangle() const;

        int getWidth() const;
        int getHeight() const;

        void clearScreen(Color col);
        void plotPixel(int x, int y, Color col);
        void drawGraphic(int x, int y, const Graphic &graphic);

        void drawLine(int x1, int y1, int x2, int y2, Color col);
        // drawRectangle is not overridden: use default implementation
        void fillRectangle(const Rectangle &rect, Color col);

        void plotPixelBatch(const Pixel *buf, int num);
        
        virtual boost::shared_ptr<PixelArray> takeScreenshot();

    private:
        ID3D11Device *device;
        ID3D11DeviceContext *device_context;
        ID3D11RenderTargetView *render_target_view;
        IDXGISwapChain *swap_chain;
        PrimitiveBatch &primitive_batch;
    };

}

#endif
